#include <ExtAPI.h>
#include "zApp.h"
#include <zClasses.h>
#include <zFunc.h>

extern CExtAPI theAPI;

bool zApp_GetTempDir(CzStr& TempDir) {
	if(!theAPI.script_Execute("zs", "application.GetTempDir", (ZVAR)TempDir)) { 
		return false;
	}
	return true;
}
long zApp_GetUID() {
	CzVar v;
	v.Create(VarID_int);
	if(!theAPI.script_Execute("zs", "application.GetUID", (ZVAR)v)) { 
		return -1;
	}
	int i = 0;
	if(!v.GetValue(VarID_int, &i)) {
		return -1;
	}
	return i;
}

bool zApp_GetPath(const char* lpPathName, CzStr& rval) {

	CzFunc f;
	if(!f.GetFunc("application.GetPath")) {
		theAPI.ReportError("zapp_GetPath error:\r\n - cannot get function");
		return false;
	}
	if(!theAPI.str_SetText(f.GetArgI(0), lpPathName)) {
		theAPI.ReportError("zapp_GetPath error:\r\n - cannot set path name string");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zapp_GetPath error:\r\n - call to application.GetPath failed");
		return false;
	}
	if(!rval.CopyValue(f.m_rval)) {
		theAPI.ReportError("zapp_GetPath error:\r\n - cannot copy rval");
		return false;
	}

	if(!rval.GetLength()) {
		// null string, indicates failure
		return false;
	}

	return true;
}

bool zApp_SetWndSetting(long DPI, const char* lpSettingName, long VarID, void* pValue) {
	
	CzFunc f;
	if(!f.GetFunc("settings.SetWndSetting")) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot get function");
		return false;
	}
	int argid = 0;
	if(!f.SetArgValueI(argid++, VarID_int, &DPI)) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot set arg");
		return false;
	}
	if(!theAPI.str_SetText(f.GetArgI(argid++), lpSettingName)) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot set arg");
		return false;
	}

	CzVar v;
	if(!v.Create(VarID, NULL)) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot create value var");
		return false;
	}
	if(!v.SetValue(VarID, pValue)) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot set value var");
		return false;
	}
	ZVAR hValue = v.GetZVAR();

	if(!f.SetArgValueI(argid++, VarID_hvar, &hValue)) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot set arg");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - function call failed");
		return false;
	}

	return true;
}
bool zApp_GetWndSetting(long DPI, const char* lpSettingName, long VarID, void* pValue, bool INF_Create, bool INF_Interp96) {
	CzFunc f;
	if(!f.GetFunc("settings.GetWndSetting")) {
		theAPI.ReportError("zapp_SetWndSetting error:\r\n - cannot get function");
		return false;
	}
	int argid = 0;
	if(!f.SetArgValueI(argid++, VarID_int, &DPI)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set arg");
		return false;
	}
	if(!theAPI.str_SetText(f.GetArgI(argid++), lpSettingName)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set arg");
		return false;
	}

	CzVar v;
	if(!v.Create(VarID, NULL)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot create value var");
		return false;
	}
	if(!v.SetValue(VarID, pValue)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set value var");
		return false;
	}
	ZVAR hValue = v.GetZVAR();
	if(!f.SetArgValueI(argid++, VarID_hvar, &hValue)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set arg");
		return false;
	}

	if(!f.SetArgValueI(argid++, VarID_bool, &INF_Create)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set arg");
		return false;
	}
	if(!f.SetArgValueI(argid++, VarID_bool, &INF_Interp96)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot set arg");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - function call failed");
		return false;
	}

	if(!v.GetValue(VarID, pValue)) {
		theAPI.ReportError("zapp_GetWndSetting error:\r\n - cannot get value var");
		return false;
	}

	return true;
}