#include <ExtAPI.h>
#include "zProj.h"
#include <zClasses.h>

extern CExtAPI theAPI;

//
// init state
//

bool zProj_IsInit() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.IsInit", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zProj_Init() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.InitProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zProj_Free() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.FreeProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}

// loading/saving project

bool zProj_LoadProject(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.LoadProject \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zProj_SaveProject() {
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.SaveProject", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zProj_SaveProjectFile() {		// just save the project file 
	CzVar v;
	v.Create(VarID_bool);
	if(!theAPI.script_Execute("zs", "project.SaveProjectFile", (ZVAR)v)) { 
		return false;
	}
	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}
	return b;
}
bool zProj_SaveProjectAs(const char* lpProjName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SaveProjectAs \"");
	s.Append(lpProjName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zProj_DemandSave() {
	CzStr s;
	if(!theAPI.script_Execute("zs", "project.SaveProjectAsUI", (ZVAR)s)) { 
		return false;
	}
	
	if(s.GetLength())
			return true;
	else	return false;
}

bool zProj_IsPathOK() { 
	if(strlen(zProj_GetProjectFilename()))
		return true;

	return false;
}
const char* zProj_GetProjectFilename() {
	return theAPI.project_GetProjectFilename();
}

//
// saving map
//

bool zProj_AutoSaveMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoSaveMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zProj_AutoLoadMap(const char* lpMapName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.AutoLoadMap \"");
	s.Append(lpMapName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

//
// accessing maps
//

ZLIST zProj_GetMapList() {

	CzFunc f;
	if(!f.GetFunc("project.GetMapListH")) {
		theAPI.ReportError("zProj_GetMapList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zProj_GetMapList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		theAPI.ReportError("zProj_GetMapList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
}
ZMAP zProj_GetMap(const char* lpMapName) {
	return theAPI.project_GetMap(lpMapName);
}
ZMAP zProj_CreateMap(const char* lpMapName) {
	return theAPI.project_CreateMap(lpMapName);
}
ZMAP zProj_GetOrCreateMap(const char* lpMapName) {
	return theAPI.project_GetOrCreateMap(lpMapName);
}
bool zProj_GetMapDisplayName(ZSTR hDest, const char* lpMapName) {

	// to be safe, set return value to be value of argument at start
	if(!theAPI.str_SetText(hDest, lpMapName)) {
		theAPI.ReportError("zProj_GetMapDisplayName error:\r\n - can't set return value");
		return false;
	}
		
	CzFunc f;
	if(!f.GetFunc("project.GetMapDisplayName")) {
		theAPI.ReportError("zProj_GetMapDisplayName error:\r\n - cannot get 'project.GetMapDisplayName' function");
		return false;
	}

	theAPI.str_SetText(f.GetArgI(0), lpMapName);

	if(!f.Execute2()) {
		theAPI.ReportError("zProj_GetMapDisplayName error:\r\n - call to 'project.GetMapDisplayName' failed");
		return false;
	}

	if(!theAPI.var_CopyValue(hDest, f.m_rval)) {
		theAPI.ReportError("zProj_GetMapDisplayName error:\r\n - can't set return value");
		return false;
	}

	return true;
}

//
// settings and options
//

ZLIST zProj_GetSettingsList() {

	CzFunc f;
	if(!f.GetFunc("project.GetSettingsListH")) {
		theAPI.ReportError("zProj_GetSettingsList error:\r\n - can't get function handle");
		return NULL;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("zProj_GetSettingsList error:\r\n - extension function failed");
		return NULL;
	}

	ZVAR hRval;
	if(!f.m_rval.GetValue(VarID_hvar, &hRval)) {
		theAPI.ReportError("zProj_GetSettingsList error:\r\n - can't get return value");
		return NULL;
	}
	return hRval;
	
}
bool zProj_GetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zProj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = theAPI.list_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!theAPI.var_GetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zProj_SetSetting(const char* lpName, long VarID, void* pValue) {
	ZLIST hSettings = zProj_GetSettingsList();
	if(!hSettings) {
		return false;
	}

	ZVAR hVar = theAPI.list_GetItemA(hSettings, lpName);
	if(!hVar) {
		return false;
	}

	if(!theAPI.var_SetValueEx(hVar, VarID, pValue)) {
		return false;
	}

	return true;
}

bool zProj_GetOpFlag(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.GetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zProj_SetOpFlag(const char* lpOpName, bool StateFlag) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.SetOpFlag \"");
	s.Append(lpOpName);
	s.Append("\" ");
	if(StateFlag)
			s.Append("true");
	else	s.Append("false");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}
bool zProj_DelOpsFollowing(const char* lpOpName) {
	CzVar v;
	v.Create(VarID_bool);

	CzStr s;
	s.SetText("project.DelOpsFollowing \"");
	s.Append(lpOpName);
	s.Append("\"");

	if(!theAPI.script_Execute("zs", (char*)s, (ZVAR)v)) { 
		return false;
	}

	bool b = false;
	if(!v.GetValue(VarID_bool, &b)) {
		return false;
	}

	return b;
}

bool zProj_GetHeightfieldRange(float& minval, float& maxval) {
	return theAPI.project_GetHeightfieldRange(minval, maxval);
}

bool zProj_SetMapFlag(const char* lpMapName, long MapFlag, bool State) {
	ZMAP m = zProj_GetMap(lpMapName);
	if(!m) {
		return false;
	}

	return theAPI.map_SetFlag(m, MapFlag, State);
}


long zProj_GetCurTileSizeDM() {
	
	CzFunc f;
	f.GetFunc("project.GetCurTileSizeDM");
	f.Execute();

	long rval;
	f.m_rval.GetValue(VarID_int, &rval);

	return rval;
}
bool zProj_SetCurTileSizeDM(long TileSizeDM) {

	CzFunc f;
	f.GetFunc("project.SetCurTileSizeDM");

	f.m_args.SetItemValueI(0, VarID_int, &TileSizeDM);
	f.Execute();

	bool rval;
	f.m_rval.GetValue(VarID_bool, &rval);
	return rval;
}

float zProj_GetCurHorizScaleHF() {

	CzFunc f;
	f.GetFunc("project.GetCurHorizScaleHF");
	f.Execute();

	float rval;
	f.m_rval.GetValue(VarID_float, &rval);

	return rval;
}
bool zProj_SetCurHorizScaleHF(float HorizScaleHF) {

	CzFunc f;
	f.GetFunc("project.SetCurHorizScaleHF");

	f.m_args.SetItemValue((long)0, VarID_float, &HorizScaleHF);
	f.Execute();

	bool rval;
	f.m_rval.GetValue(VarID_bool, &rval);
	return rval;
}

int zProj_nClimates() {
	CzFunc f;
	if(!f.GetFunc("project.nClimates")) {
		return 0;
	}
	if(!f.Execute2()) {
		return 0;
	}

	int rval = 0;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		return 0;
	}
	return rval;
}
ZVAR zProj_GetClimate(int CliID) {
	CzFunc f;
	f.GetFunc("project.GetClimate");
	f.SetArgValueI(0, VarID_int, &CliID);
	if(!f.Execute2())	return NULL;

	ZVAR hCli = NULL;
	f.m_rval.GetValue(VarID_hvar, &hCli);
	return hCli;
}
ZVAR zProj_GetClimateA(const char* lpCliName) {
	CzFunc f;
	f.GetFunc("project.GetClimateA");
	theAPI.str_SetText(f.GetArgI(0), lpCliName);
	if(!f.Execute2())	return NULL;

	ZVAR hCli = NULL;
	f.m_rval.GetValue(VarID_hvar, &hCli);
	return hCli;
}
int zProj_GetClimateIndex(const char* lpCliName) {
	CzFunc f;
	f.GetFunc("project.GetClimateIndex");
	theAPI.str_SetText(f.GetArgI(0), lpCliName);
	if(!f.Execute2())	return -1;

	int rval = -1;
	f.m_rval.GetValue(VarID_int, &rval);
	return rval;
}
int zProj_AddClimateA(const char* lpCliName) {
	CzFunc f;
	f.GetFunc("project.AddClimateToProject");
	theAPI.str_SetText(f.GetArgI(0), lpCliName);
	if(!f.Execute2())	return -1;

	int rval = -1;
	f.m_rval.GetValue(VarID_int, &rval);
	return rval;
}