#include "zvarcast.h"
#include <stdafx.h>

#include <ExtAPI.h>
extern CExtAPI theAPI;

#include <zFunc.h>
#include <zList.h>
#include <zVar.h>
#include <zStr.h>

bool ZeoVarCast(ZVAR hVarDest, long VarID, ZVAR hVarToCast) {

	ZVAR hTempVar = NULL;

	switch(VarID) {
	case VarID_INVALID:
		return false; // invalid!
	case VarID_void:
		return false; // can't cast to void
	case VarID_variant:
		return false; // not supported yet
	case VarID_bool:
		return ZeoVarCastBool(hVarDest, hVarToCast);
	case VarID_char:
		return false; // not supported yet
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
		return ZeoVarCastInt(hVarDest, VarID, hVarToCast);
	case VarID_float:
	case VarID_double:
		return ZeoVarCastFloat(hVarDest, VarID, hVarToCast);
	case VarID_string:
		return ZeoVarCastString(hVarDest, hVarToCast);
	case VarID_colour:
	case VarID_vector3c:
	case VarID_vector3i:
	case VarID_vector3f:
	case VarID_vector3d:
	case VarID_buffer:
	case VarID_voidptr:
	case VarID_hvar:
	case VarID_varid:
	case VarID_varlist:
	case VarID_map:
	case VarID_format:
	case VarID_FileSelector:
	case VarID_DirSelector:
	case VarID_ComboSelector:
	case VarID_Climate:
	case VarID_LandType:
	case VarID_ZeoFunc:
	case VarID_ProgBox:
	default:
		return false; // not supported yet
	}

	return false;
}

bool ZeoVarCastBool(ZVAR hVarDest, ZVAR hVarToCast) {

	bool rval = false;

	BYTE b;
	short s;
	int i;
	__int64 i64;
	float f;
	double d;
	LPVOID lp;
	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = theAPI.var_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		theAPI.type_GetTypeName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastBool error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		theAPI.ReportError(TempStr);
		return false;
	case VarID_variant:
		hTempVar = theAPI.var_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			theAPI.ReportError("ZeoVarCastBool error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!ZeoVarCastBool(hVarDest, hTempVar)) {
			theAPI.ReportError("ZeoVarCastBool error:\r\n - call to VarCastBool failed for variant object");
			return false;
		}
		return true;
	case VarID_bool:
		if(!theAPI.var_GetValueEx(hVarToCast, VarID_bool, &rval)) {
			return false;
		}
		break;
	case VarID_char:
	case VarID_byte:
	case VarID_sbyte:
		if(!theAPI.var_GetValue(hVarToCast, &b)) {
			return false;
		}
		if(b)	rval = true;
		else	rval = false;
		break;
	case VarID_short:
	case VarID_ushort:
		if(!theAPI.var_GetValue(hVarToCast, &s)) {
			return false;
		}
		if(s)	rval = true;
		else	rval = false;
		break;
	case VarID_int:
	case VarID_uint:
	case VarID_varid:
		if(!theAPI.var_GetValue(hVarToCast, &i)) {
			return false;
		}
		if(i)	rval = true;
		else	rval = false;
		break;
	case VarID_int64:
		if(!theAPI.var_GetValue(hVarToCast, &i64)) {
			return false;
		}
		if(i64)	rval = true;
		else	rval = false;
		break;
	case VarID_float:
		if(!theAPI.var_GetValue(hVarToCast, &f)) {
			return false;
		}
		if(f==0.0f)
				rval = false;
		else	rval = true;
		break;
	case VarID_double:
		if(!theAPI.var_GetValue(hVarToCast, &d)) {
			return false;
		}
		if(d==0.0f)
				rval = false;
		else	rval = true;
		break;
	case VarID_voidptr:
		if(!theAPI.var_GetValue(hVarToCast, &lp)) {
			return false;
		}
		if(lp)	rval = true;
		else	rval = false;
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = theAPI.var_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_bool:
		if(!theAPI.var_SetValue(hVarDest, &rval)) {
			theAPI.ReportError("ZeoVarCastBool error:\r\n - cannot set value (bool case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = theAPI.var_GetDataPtr(hVarDest))) {
			theAPI.ReportError("ZeoVarCastBool error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!theAPI.var_IsType(hVar2, VarID_bool)) {
			if(!theAPI.var_ReInit(hVar2, VarID_bool)) {
				theAPI.ReportError("ZeoVarCastBool error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		if(!theAPI.var_SetValue(hVar2, &rval)) {
			theAPI.ReportError("ZeoVarCastBool error:\r\n - cannot set value (variant case case)");
			return false;
		}
		break;
	default:
		// un-supported variable type
		theAPI.type_GetTypeName(DestVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastBool error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		theAPI.ReportError(TempStr);
		return false;
	}


	return true;
}
bool ZeoVarCastInt(ZVAR hVarDest, long VarID, ZVAR hVarToCast) {
	
	__int64 rval = 0;

	switch(VarID) {
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
		break;
	default:
		theAPI.ReportError("ZeoVarCastInt error:\r\n - unsupported output VarID");
		return false;
	}

	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = theAPI.var_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		theAPI.type_GetTypeName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastInt error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		theAPI.ReportError(TempStr);
		return false;
	case VarID_variant:
		hTempVar = theAPI.var_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!ZeoVarCastInt(hVarDest, VarID, hTempVar)) {
			theAPI.ReportError("ZeoVarCastInt error:\r\n - call to VarCastInt failed for variant object");
			return false;
		}
		return true;
	case VarID_byte:
		{
			BYTE v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_sbyte:
		{
			char v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_short:
		{
			short v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_ushort:
		{
			unsigned short v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_int:
	case VarID_varid:
		{
			int v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_uint:
		{
			unsigned int v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_int64:
		if(!theAPI.var_GetValue(hVarToCast, &rval)) {
			return false;
		}
		break;
	case VarID_float:
		{
			float v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_double:
		{
			double v;
			if(!theAPI.var_GetValue(hVarToCast, &v)) {
				return false;
			}
			rval = (__int64)v;
		}
		break;
	case VarID_string:
		rval = _atoi64(theAPI.str_GetText(hVarToCast));
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = theAPI.var_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_byte:
		{
			unsigned  v = (unsigned char)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (byte case)");
				return false;
			}
		}
		break;
	case VarID_sbyte:
		{
			char v = (char)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (sbyte case)");
				return false;
			}
		}
		break;
	case VarID_short:
		{
			short v = (short)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (short case)");
				return false;
			}
		}
		break;
	case VarID_ushort:
		{
			unsigned short v = (unsigned short)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (ushort case)");
				return false;
			}
		}
		break;
	case VarID_int:
		{
			int v = (int)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (int case)");
				return false;
			}
		}
		break;
	case VarID_uint:
		{
			unsigned int v = (unsigned int)rval;
			if(!theAPI.var_SetValue(hVarDest, &v)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (uint case)");
				return false;
			}
		}
		break;
	case VarID_int64:
		if(!theAPI.var_SetValue(hVarDest, &rval)) {
			theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (__int64 case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = theAPI.var_GetDataPtr(hVarDest))) {
			theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!theAPI.var_IsType(hVar2, VarID)) {
			if(!theAPI.var_ReInit(hVar2, VarID)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		switch(VarID) {
		case VarID_byte:
			{
				unsigned char v = (unsigned char)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/byte case)");
					return false;
				}
			}
			break;
		case VarID_sbyte:
			{
				char v = (char)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/sbyte case)");
					return false;
				}
			}
			break;
		case VarID_short:
			{
				short v = (short)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/short case)");
					return false;
				}
			}
			break;
		case VarID_ushort:
			{
				unsigned short v = (unsigned short)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/ushort case)");
					return false;
				}
			}
			break;
		case VarID_int:
			{
				int v = (int)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/int case)");
					return false;
				}
			}
			break;
		case VarID_uint:
			{
				unsigned int v = (unsigned int)rval;
				if(!theAPI.var_SetValue(hVar2, &v)) {
					theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/uint case)");
					return false;
				}
			}
			break;
		case VarID_int64:
			if(!theAPI.var_SetValue(hVar2, &rval)) {
				theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/__int64 case)");
				return false;
			}
			break;
		default:
			theAPI.ReportError("ZeoVarCastInt error:\r\n - cannot set value (variant/unknown case)");
			return false;
		}
		
		break;
	default:
		// un-supported variable type
		theAPI.type_GetTypeName(DestVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastInt error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		theAPI.ReportError(TempStr);
		return false;
	}


	return true;
}
bool ZeoVarCastFloat(ZVAR hVarDest, long VarID, ZVAR hVarToCast) {
	
	double rval = 0.0;

	switch(VarID) {
	case VarID_float:
	case VarID_double:
		break;
	default:
		theAPI.ReportError("ZeoVarCastFloat error:\r\n - unsupported output VarID");
		return false;
	}

	BYTE b;
	char c;
	short s;
	unsigned short us;
	int i;
	unsigned int ui;
	__int64 i64;
	float f;
	CStringA TempStr;
	CzStr zStr;
	ZVAR hTempVar;
	
	long SrcVarID = theAPI.var_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		theAPI.type_GetTypeName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastFloat error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		theAPI.ReportError(TempStr);
		return false;
	case VarID_variant:
		hTempVar = theAPI.var_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!ZeoVarCastFloat(hVarDest, VarID, hTempVar)) {
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - call to VarCastFloat failed for variant object");
			return false;
		}
		return true;
	case VarID_byte:
		if(!theAPI.var_GetValue(hVarToCast, &b)) {
			return false;
		}
		rval = (double)b;
		break;
	case VarID_sbyte:
		if(!theAPI.var_GetValue(hVarToCast, &c)) {
			return false;
		}
		rval = (double)c;
		break;
	case VarID_short:
		if(!theAPI.var_GetValue(hVarToCast, &s)) {
			return false;
		}
		rval = (double)s;
		break;
	case VarID_ushort:
		if(!theAPI.var_GetValue(hVarToCast, &us)) {
			return false;
		}
		rval = (double)us;
		break;
	case VarID_int:
		if(!theAPI.var_GetValue(hVarToCast, &i)) {
			return false;
		}
		rval = (double)i;
		break;
	case VarID_uint:
		if(!theAPI.var_GetValue(hVarToCast, &ui)) {
			return false;
		}
		rval = (double)ui;
		break;
	case VarID_int64:
		if(!theAPI.var_GetValue(hVarToCast, &i64)) {
			return false;
		}
		rval = (double)i64;
		break;
	case VarID_float:
		if(!theAPI.var_GetValue(hVarToCast, &f)) {
			return false;
		}
		rval = (double)f;
		break;
	case VarID_double:
		if(!theAPI.var_GetValue(hVarToCast, &rval)) {
			return false;
		}
		break;
	case VarID_string:
		rval = atof(theAPI.str_GetText(hVarToCast));
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = theAPI.var_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_float:
		f = (float)rval;
		if(!theAPI.var_SetValue(hVarDest, &f)) {
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot set value (float case)");
			return false;
		}
		break;
	case VarID_double:
		if(!theAPI.var_SetValue(hVarDest, &rval)) {
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot set value (double case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = theAPI.var_GetDataPtr(hVarDest))) {
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!theAPI.var_IsType(hVar2, VarID)) {
			if(!theAPI.var_ReInit(hVar2, VarID)) {
				theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		switch(VarID) {
		case VarID_float:
			f = (float)rval;
			if(!theAPI.var_SetValue(hVar2, &f)) {
				theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot set value (variant/float case)");
				return false;
			}
			break;
		case VarID_double:
			if(!theAPI.var_SetValue(hVar2, &rval)) {
				theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot set value (variant/double case)");
				return false;
			}
			break;
		default:
			theAPI.ReportError("ZeoVarCastFloat error:\r\n - cannot set value (variant/unknown case)");
			return false;
		}
		
		break;
	default:
		// un-supported variable type
		theAPI.type_GetTypeName(DestVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastFloat error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		theAPI.ReportError(TempStr);
		return false;
	}


	return true;
}
bool ZeoVarCastString(ZVAR hVarDest, ZVAR hVarToCast) {
	
	CStringA rval = "";

	CzStr zStr;
	CStringA TempStr;
	ZVAR hTempVar;
	
	long SrcVarID = theAPI.var_GetTypeID(hVarToCast);
	switch(SrcVarID) {
	default:
		theAPI.type_GetTypeName(SrcVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastString error:\r\n - unsupported input variable type '%s'/%d", (char*)zStr, SrcVarID);
		theAPI.ReportError(TempStr);
		return false;
	case VarID_variant:
		hTempVar = theAPI.var_GetDataPtr(hVarToCast);
		if(!hTempVar) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - cannot retrieve variant src data");
			return false;
		}
		if(!ZeoVarCastString(hVarDest, hTempVar)) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - call to VarCastString failed for variant object");
			return false;
		}
		return true;
	case VarID_bool:
	case VarID_char:
	case VarID_byte:
	case VarID_sbyte:
	case VarID_short:
	case VarID_ushort:
	case VarID_int:
	case VarID_uint:
	case VarID_int64:
	case VarID_float:
	case VarID_double:
	case VarID_voidptr:
	case VarID_varid:
	case VarID_string:
	case VarID_buffer:
	case VarID_colour:
	case VarID_vector3c:
	case VarID_vector3i:
	case VarID_vector3f:
	case VarID_vector3d:
		if(!theAPI.var_GetValueA(hVarToCast, (ZVAR)zStr)) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - call to var_GetValueA failed");
			return false;
		}
		rval.SetString((char*)zStr);
		break;
	/*case VarID_hVar:
		return false; // not supported yet
		*/
	}

	// now set rval
	ZVAR hVar2 = NULL; // used for variant value setting
	int DestVarID = theAPI.var_GetTypeID(hVarDest);
	switch(DestVarID) {
	case VarID_string:
		if(!theAPI.var_SetValue(hVarDest, &rval)) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - cannot set value (string case)");
			return false;
		}
		break;
	case VarID_variant:
		if(!(hVar2 = theAPI.var_GetDataPtr(hVarDest))) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - cannot retrieve variant data");
			return false;
		}
		if(!theAPI.var_IsType(hVar2, VarID_string)) {
			if(!theAPI.var_ReInit(hVar2, VarID_string)) {
				theAPI.ReportError("ZeoVarCastString error:\r\n - cannot reinitialise variant data");
				return false;
			}
		}
		if(!theAPI.var_SetValue(hVar2, &rval)) {
			theAPI.ReportError("ZeoVarCastString error:\r\n - cannot set value (variant case)");
			return false;
		}
		break;
	default:
		// un-supported variable type
		theAPI.type_GetTypeName(DestVarID, (ZSTR)zStr);
		TempStr.Format("ZeoVarCastString error:\r\n - unsupported output variable type '%s'/%d", (char*)zStr, DestVarID);
		theAPI.ReportError(TempStr);
		return false;
	}

	return true;
}