#include "zclimate.h"
#include "zfunc.h"
#include "zstr.h"
#include "extapi.h"
extern CExtAPI theAPI;

CzClimate::CzClimate(void) {
	m_hTypeList = NULL;
}

CzClimate::~CzClimate(void) {
}

bool CzClimate::Attach(ZVAR hVar) {

	if(VarID_Climate!=theAPI.var_GetTypeID(hVar)) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - no climate var attached");
		return false;
	}

	//
	// now get land type list
	//
	CzFunc f;
	if(!f.GetFunc("classes.climate.GetTypeList")) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - cannot get func handle");
		return false;
	}
	if(!f.SetArgValueI(0, VarID_hvar, &hVar)) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - cannot set arg value");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - call to func failed");
		return false;
	}
	m_hTypeList = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &m_hTypeList)) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - cannot get return value");
		return false;
	}

	// now finally attach to var
	if(!CzVar::Attach(hVar)) {
		theAPI.ReportError("CzClimate::Attach error:\r\n - call to CzVar::Attach failed");
		return false;
	}

	return true;
}

long CzClimate::nTypes() {	return theAPI.list_nItems(m_hTypeList); }

ZVAR CzClimate::GetType(long index) {
	return theAPI.list_GetItemI(m_hTypeList, index);
}

bool CzClimate::GetClimateName(ZSTR hNameVar) {
	
	if(!m_hVar) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - no climate var attached");
		return false;
	}

	if(!theAPI.var_IsType(hNameVar, VarID_string)) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - argument is invalid");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("classes.climate.GetName")) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - cannot get func handle");
		return false;
	}
	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - cannot set arg value");
		return false;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - call to func fauled");
		return false;
	}
	if(!theAPI.var_CopyValue(hNameVar, (ZVAR)f.m_rval)) {
		theAPI.ReportError("CzClimate::GetClimateName error:\r\n - cannot copy rval");
		return false;
	}

	return true;
}

long CzClimate::AddToProject() {
	if(!theAPI.var_IsType(m_hVar, VarID_Climate)) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - no climate var attached");
		return -1;
	}
	
	CzFunc f;
	if(!f.GetFunc("project.AddClimateToProject")) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - cannot get func handle");
		return -1;
	}

	CzStr CliName;
	if(!GetClimateName((ZVAR)CliName)) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - cannot get climate name");
		return -1;
	}

	if(!theAPI.var_CopyValue(f.GetArgI(0), (ZVAR)CliName)) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - call to func failed");
		return false;
	}

	long rval;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzClimate::AddToProject error:\r\n - cannot get return value");
		return -1;
	}

	return rval;
}

// getting climate IDs in application and project
long CzClimate::GetProjectID() {
	
	if(!theAPI.var_IsType(m_hVar, VarID_Climate)) {
		theAPI.ReportError("CzClimate::GetProjectID error:\r\n - no climate var attached");
		return -1;
	}

	CzFunc f;
	if(!f.GetFunc("classes.climate.GetProjID")) {
		theAPI.ReportError("CzClimate::GetProjectID error:\r\n - cannot get func handle");
		return -1;
	}

	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		theAPI.ReportError("CzClimate::GetProjectID error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzClimate::GetProjectID error:\r\n - call to func fauled");
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzClimate::GetProjectID error:\r\n - cannot get rval");
		return -1;
	}

	return rval;
}
long CzClimate::GetApplicationID() {
	if(!theAPI.var_IsType(m_hVar, VarID_Climate)) {
		theAPI.ReportError("CzClimate::GetApplicationID error:\r\n - no climate var attached");
		return -1;
	}

	CzFunc f;
	if(!f.GetFunc("classes.climate.GetAppID")) {
		theAPI.ReportError("CzClimate::GetApplicationID error:\r\n - cannot get func handle");
		return -1;
	}

	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		theAPI.ReportError("CzClimate::GetApplicationID error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		theAPI.ReportError("CzClimate::GetApplicationID error:\r\n - call to func fauled");
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		theAPI.ReportError("CzClimate::GetApplicationID error:\r\n - cannot get rval");
		return -1;
	}

	return rval;
}